library sadad_payment_flutter;

import 'dart:convert';
import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:sadad_payment_flutter/apputils/appassets.dart';
import 'package:sadad_payment_flutter/apputils/appcolors.dart';
import 'package:sadad_payment_flutter/commonWidgets.dart';
import 'package:sadad_payment_flutter/controller.dart';

import 'package:sadad_payment_flutter/services/appservices.dart';

class PaymentScreen extends StatefulWidget {
  Color? themeColor;
  Color? paymentButtonColor;
  Color? paymentButtonTextColor;
  DevicePlatform? devicePlatform;
  List<PaymentType>? paymentTypes;
  String token;
  Image? image;
  PackageMode? packageMode;

  PaymentScreen(
      {super.key,
      this.themeColor = const Color(0xFFA02058),
      this.devicePlatform = DevicePlatform.applePay,
      this.paymentButtonColor,
      this.paymentButtonTextColor,required this.token,
      this.packageMode = PackageMode.debug,
      this.image,
      this.paymentTypes = const [PaymentType.creditCard, PaymentType.debitCard, PaymentType.sadadPay]});

  @override
  State<PaymentScreen> createState() => _PaymentScreenState();
}

class _PaymentScreenState extends State<PaymentScreen> {
  Color primaryColor = AppColors.primaryColor;
  MainPageController controller = Get.put(MainPageController());

  @override
  void initState() {
    AppServices.getAccessToken();
    super.initState();
    primaryColor = widget.themeColor!;
    if (widget.paymentTypes!.isEmpty) {
      widget.paymentTypes = [PaymentType.creditCard, PaymentType.debitCard, PaymentType.sadadPay];
    } else {
      controller.selectedPaymentMethod = widget.paymentTypes!.first;
    }
  }

  @override
  Widget build(BuildContext context) {
    return GetBuilder(
        init: MainPageController(),
        builder: (MainPageController controller) {
          return GestureDetector(
            onTap: () {
              FocusScope.of(context).unfocus();
            },
            child: Scaffold(
                backgroundColor: const Color(0xFFFAFAFA),
                body: SafeArea(
                    child: SingleChildScrollView(
                  child: Column(
                    children: [topContainer(), middleContainer(), bottomContainer()],
                  ),
                ))),
          );
        });
  }

  Widget middleContainer() {
    MainPageController controller = Get.find<MainPageController>();
    return Padding(
      padding: const EdgeInsets.all(12),
      child: Column(
        children: [
          if (widget.paymentTypes!.contains(PaymentType.wallet))
            Row(
              children: const [
                Expanded(
                  child: Divider(
                    endIndent: 15,
                    indent: 15,
                    thickness: 2,
                    height: 10,
                    color: Color(0xFFE4E4E4),
                  ),
                ),
                Text("Or Pay Using", style: TextStyle(color: AppColors.black, fontSize: 12)),
                Expanded(
                  child: Divider(
                    endIndent: 15,
                    indent: 15,
                    thickness: 2,
                    height: 10,
                    color: AppColors.dividerColor,
                  ),
                )
              ],
            ),
          const SizedBox(height: 12),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              if (widget.paymentTypes!.contains(PaymentType.creditCard)) cardContainer(text: "Credit Card", type: PaymentType.creditCard, imagePath: AssetPath.creditCard),
              if (widget.paymentTypes!.contains(PaymentType.debitCard)) cardContainer(text: "Debit Card", type: PaymentType.debitCard, imagePath: AssetPath.cardNaps),
              if (widget.paymentTypes!.contains(PaymentType.sadadPay)) cardContainer(text: "Sadad Pay", type: PaymentType.sadadPay, imagePath: AssetPath.sadadWallet)
            ],
          ),
          const SizedBox(height: 12),
          getSelectedPaymentMethod(selectedMethod: controller.selectedPaymentMethod)
        ],
      ),
    );
  }

  Widget topContainer() {
    return SizedBox(
      height: Get.height * 0.25,
      width: double.infinity,
      child: Stack(children: [
        Container(
          height: Get.height * 0.24,
          decoration: BoxDecoration(color: primaryColor, borderRadius: const BorderRadius.only(bottomLeft: Radius.circular(60), bottomRight: Radius.circular(60))),
          child: Padding(
            padding: const EdgeInsets.all(12),
            child: Column(mainAxisAlignment: MainAxisAlignment.spaceBetween, children: [
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  const Expanded(
                    child: Align(
                      alignment: Alignment.centerLeft,
                      child: Icon(
                        Icons.arrow_back_ios_new_outlined,
                        color: Colors.white,
                      ),
                    ),
                  ),
                  widget.image ?? const SizedBox(),
                  Expanded(
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.end,
                      children: const [Text("Eng", style: TextStyle(color: Colors.white, fontSize: 12)), Icon(Icons.keyboard_arrow_down_rounded, color: AppColors.white)],
                    ),
                  )
                ],
              ),
              Container(
                  padding: const EdgeInsets.symmetric(horizontal: 16),
                  decoration: BoxDecoration(color: AppColors.black54, borderRadius: BorderRadius.circular(20)),
                  height: 40,
                  child: Row(mainAxisSize: MainAxisSize.min, children: const [
                    Text(
                      "Select an option to Pay ",
                      style: TextStyle(color: AppColors.white, fontSize: 12),
                    ),
                    Text("QAR 25555", style: TextStyle(color: AppColors.white, fontSize: 15, fontWeight: FontWeight.w900))
                  ])),
              const SizedBox(height: 25)
            ]),
          ),
        ),
        if (widget.paymentTypes!.contains(PaymentType.wallet))
          widget.packageMode == PackageMode.debug
              ? Container(color: widget.themeColor, height: 20, width: 20)
              : Align(
                  alignment: Alignment.bottomCenter,
                  child: widget.devicePlatform == DevicePlatform.applePay
                      ? Container(
                          decoration: BoxDecoration(color: AppColors.black, borderRadius: BorderRadius.circular(5)),
                          height: 55,
                          margin: const EdgeInsets.symmetric(horizontal: 20),
                          child: Center(child: Image.asset(AssetPath.applePay, package: 'sadad_payment_flutter', height: 22)),
                        )
                      : Container(
                          decoration: BoxDecoration(
                              color: AppColors.white, borderRadius: BorderRadius.circular(5), boxShadow: const [BoxShadow(color: AppColors.black12, blurRadius: 5, blurStyle: BlurStyle.outer)]),
                          height: 55,
                          margin: const EdgeInsets.symmetric(horizontal: 20),
                          child: Center(child: Image.asset(AssetPath.gPay, package: 'sadad_payment_flutter', height: 22)),
                        ),
                )
      ]),
    );
  }

  Widget cardContainer({required String text, required String imagePath, required PaymentType type}) {
    MainPageController controller = Get.find<MainPageController>();
    return Expanded(
      child: GestureDetector(
        onTap: () {
          controller.selectedPaymentMethod = type;
          controller.update();
        },
        child: Container(
          margin: EdgeInsets.symmetric(horizontal: 10),
          clipBehavior: Clip.hardEdge,
          foregroundDecoration:
              BoxDecoration(borderRadius: BorderRadius.circular(5), boxShadow: const [BoxShadow(color: AppColors.boxShadow, offset: Offset(0, 3), blurStyle: BlurStyle.inner, blurRadius: 5)]),
          height: 60,
          decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), color: AppColors.white, boxShadow: const [BoxShadow(color: Colors.black12, blurRadius: 5, blurStyle: BlurStyle.outer)]),
          child: Stack(
            alignment: Alignment.bottomCenter,
            children: [
              Padding(
                padding: const EdgeInsets.only(left: 16, right: 16, top: 6),
                child: Column(mainAxisAlignment: MainAxisAlignment.spaceBetween, crossAxisAlignment: CrossAxisAlignment.center, children: [
                  widget.packageMode == PackageMode.debug
                      ? Container(color: widget.themeColor, height: 20, width: 20)
                      : ImageIcon(AssetImage(imagePath, package: 'sadad_payment_flutter'), size: 25, color: controller.selectedPaymentMethod == type ? primaryColor : AppColors.grey),
                  FittedBox(fit: BoxFit.scaleDown, child: Text(text, style: const TextStyle(fontSize: 12, fontWeight: FontWeight.w500))),
                  Container(height: 8)
                ]),
              ),
              Container(height: 8, color: controller.selectedPaymentMethod == type ? primaryColor : AppColors.transparent)
            ],
          ),
        ),
      ),
    );
  }

  getSelectedPaymentMethod({required PaymentType selectedMethod}) {
    switch (selectedMethod) {
      case PaymentType.creditCard:
        return creditCardContainer();
      case PaymentType.debitCard:
        return debitCardContainer();
      case PaymentType.sadadPay:
        return sadadPayContainer();
      case PaymentType.wallet:
        // TODO: Handle this case.
        break;
    }
  }

  Widget creditCardContainer() {
    return Container(
      height: 405,
      margin: const EdgeInsets.all(10),
      padding: const EdgeInsets.all(20),
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: AppColors.white,
          border: Border.all(color: AppColors.borderColor),
          boxShadow: const [BoxShadow(color: AppColors.black12, offset: Offset(0, 0), blurRadius: 5, blurStyle: BlurStyle.outer)]),
      child: Column(children: [
        Row(
          children: [
            widget.packageMode == PackageMode.debug
                ? Container(height: 20, width: 20, color: widget.themeColor)
                : ImageIcon(const AssetImage(AssetPath.creditCard, package: 'sadad_payment_flutter'), color: widget.themeColor),
            const SizedBox(width: 16),
            Text(
              "Credit Card",
              style: TextStyle(color: primaryColor, fontSize: 15, fontWeight: FontWeight.bold),
            ),
          ],
        ),
        const SizedBox(height: 15),
        CommonWidgets.commonUnderLineTextField(controller: controller.cardHolderNameController, labelText: "Card Holder Name", themeColor: widget.themeColor ?? primaryColor,maxlength: 100),
        CommonWidgets.commonUnderLineTextField(
            controller: controller.cardNumberController,
            maxlength: 12,
            labelText: "Card Number",
            onChanged: (cardNumber) {

            },
            //onChanged:
            suffixIcon:
                widget.packageMode == PackageMode.debug ? Container(height: 20, width: 20, color: widget.themeColor) : Image.asset(AssetPath.card2, package: 'sadad_payment_flutter', height: 19),
            themeColor: widget.themeColor ?? primaryColor),
        Row(
          children: [
            Expanded(
              child: CommonWidgets.commonUnderLineTextField(controller: controller.expiryController, labelText: "Expiry / Validity", themeColor: widget.themeColor ?? primaryColor,maxlength: 4),
            ),
            const SizedBox(width: 15),
            Expanded(
              child: CommonWidgets.commonUnderLineTextField(
                  controller: controller.cvvController,
                  maxlength: 3,
                  labelText: "CVV",
                  suffixIcon:
                      widget.packageMode == PackageMode.debug ? Container(height: 20, width: 20, color: widget.themeColor) : Image.asset(AssetPath.card1, package: 'sadad_payment_flutter', height: 26),
                  themeColor: widget.themeColor ?? primaryColor),
            ),
          ],
        ),
        Row(
          children: [
            SizedBox(
                height: 24.0,
                width: 24.0,
                child: Checkbox(
                  value: controller.rememberMe,
                  onChanged: (value) {
                    controller.rememberMe = value!;
                    controller.update();
                  },
                  activeColor: AppColors.green,
                )),
            const SizedBox(width: 8),
            Text(
              "Remember me test",
              style: TextStyle(fontSize: 10, color: AppColors.black.withOpacity(0.5)),
            )
          ],
        ),
        const Spacer(),
        paymentButton(),
        const SizedBox(height: 15),
        commonSecurityText()
      ]),
    );
  }

  Widget sadadPayContainer() {
    return Container(
      height: 405,
      margin: const EdgeInsets.all(10),
      padding: const EdgeInsets.all(20),
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: Colors.white,
          border: Border.all(color: AppColors.borderColor),
          boxShadow: const [BoxShadow(color: AppColors.black12, offset: Offset(0, 0), blurRadius: 5, blurStyle: BlurStyle.outer)]),
      child: Column(children: [
        Row(
          children: [
            widget.packageMode == PackageMode.debug
                ? Container(height: 20, width: 20, color: widget.themeColor)
                : ImageIcon(const AssetImage(AssetPath.sadadWallet, package: 'sadad_payment_flutter'), color: widget.themeColor),
            const SizedBox(width: 16),
            Text(
              "Sadad Pay",
              style: TextStyle(color: primaryColor, fontSize: 15, fontWeight: FontWeight.bold),
            ),
          ],
        ),
        const SizedBox(height: 15),
        CommonWidgets.commonTextField(controller: controller.cellNumberController, themeColor: widget.themeColor ?? primaryColor, labelText: "Cell Number"),
        CommonWidgets.commonTextField(
            controller: controller.sadadPayPasswordController,
            obscure: controller.isObscure,
            themeColor: widget.themeColor ?? primaryColor,
            labelText: "Password",
            suffixIcon: Padding(
              padding: const EdgeInsets.all(12),
              child: GestureDetector(
                onTap: () {
                  if (controller.isObscure) {
                    controller.isObscure = false;
                  } else {
                    controller.isObscure = true;
                  }
                  controller.update();
                },
                child: widget.packageMode == PackageMode.debug
                    ? Container(height: 20, width: 20, color: widget.themeColor)
                    : Image.asset(controller.isObscure ? AssetPath.icEyeClose : AssetPath.icEyeOpen, package: 'sadad_payment_flutter', color: widget.themeColor, height: 20),
              ),
            )),
        const Spacer(),
        paymentButton(),
        const SizedBox(height: 15),
        commonSecurityText()
      ]),
    );
  }

  Widget debitCardContainer() {
    return Container(
      height: 405,
      margin: const EdgeInsets.all(10),
      padding: const EdgeInsets.all(20),
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: Colors.white,
          border: Border.all(color: AppColors.borderColor),
          boxShadow: const [BoxShadow(color: AppColors.black12, offset: Offset(0, 0), blurRadius: 5, blurStyle: BlurStyle.outer)]),
      child: Column(children: [
        Row(
          children: [
            widget.packageMode == PackageMode.debug
                ? Container(height: 20, width: 20, color: widget.themeColor)
                : ImageIcon(const AssetImage(AssetPath.cardNaps, package: 'sadad_payment_flutter'), color: widget.themeColor),
            const SizedBox(width: 16),
            Text(
              "Debit Card",
              style: TextStyle(color: primaryColor, fontSize: 15, fontWeight: FontWeight.bold),
            ),
          ],
        ),
        const SizedBox(height: 15),
        const Text(
          "When you choose to complete your payment using a debit card issued in Qatar, you will be temporarily redirected to the QPay website. Once the transaction is completed, you'll be taken back to the Sadad to view your confirmation.",
          style: TextStyle(fontFamily: "openSans", fontSize: 12, fontWeight: FontWeight.w400),
        ),
        const Spacer(),
        paymentButton(),
        const SizedBox(height: 15),
        commonSecurityText()
      ]),
    );
  }

  bottomContainer() {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 8.0),
      child: Column(
        children: [
          Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: [securityContainer(AssetPath.mcAfee), securityContainer(AssetPath.comodoSecure), securityContainer(AssetPath.symantec), securityContainer(AssetPath.pciDss)]),
          const SizedBox(height: 15),
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              widget.packageMode == PackageMode.debug
                  ? Container(height: 20, width: 20, color: widget.themeColor)
                  : ImageIcon(const AssetImage(AssetPath.lock, package: 'sadad_payment_flutter'), size: 10, color: widget.themeColor),
              const SizedBox(width: 3),
              Text("Payment is Secured with 256bit SSL encryption ", style: TextStyle(fontSize: 10, color: AppColors.greyTextColor)),
              Text(
                "(You are safe)",
                style: TextStyle(fontSize: 10, color: primaryColor),
              )
            ],
          ),
          const SizedBox(height: 15)
        ],
      ),
    );
  }

  commonSecurityText() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        widget.packageMode == PackageMode.debug
            ? Container(height: 20, width: 20, color: widget.themeColor)
            : ImageIcon(const AssetImage(AssetPath.lock, package: 'sadad_payment_flutter'), size: 10, color: widget.themeColor),
        const SizedBox(width: 3),
        Expanded(
          child: Wrap(children: [
            Text("Payment is Secured with 256bit SSL encryption ", style: TextStyle(fontSize: 10, color: AppColors.greyTextColor)),
            Text(
              "(You are safe)",
              style: TextStyle(fontSize: 10, color: primaryColor),
            )
          ]),
        )
      ],
    );
  }

  Widget securityContainer(String imagepath) {
    return Container(
      height: 50,
      padding: const EdgeInsets.all(5),
      decoration: BoxDecoration(color: AppColors.borderColor, borderRadius: BorderRadius.circular(5)),
      child: widget.packageMode == PackageMode.debug ? Container(height: 20, width: 20, color: widget.themeColor) : Image.asset(imagepath, width: Get.width * 0.20, package: 'sadad_payment_flutter'),
    );
  }

  Widget paymentButton() {
    return InkWell(
        onTap: () {
          var cardType = controller.getCardType(controller.cardNumberController.text);

          setState(() {});
        },
        child: Container(
          decoration: BoxDecoration(color: widget.paymentButtonColor ?? primaryColor, borderRadius: BorderRadius.circular(10), border: Border.all(color: AppColors.paymentButtonBorder)),
          height: 50,
          child: Center(
              child: Text(
            "Pay QAR 950",
            style: TextStyle(color: widget.paymentButtonTextColor ?? Colors.white, fontSize: 17, fontWeight: FontWeight.w800),
          )),
        ));
  }
}

enum DevicePlatform { applePay, googlePay }

enum PaymentType { creditCard, debitCard, sadadPay, wallet }

enum PackageMode { release, debug }
